﻿using System;
using System.ComponentModel;
using Wisej.Web;

namespace WisejWebApplication
{
    public partial class ListViewBinding : Wisej.Web.ListView
    {
        protected new bool DesignMode => base.DesignMode || LicenseManager.UsageMode == LicenseUsageMode.Designtime;


        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public bool AutoGenerateColumns { get; set; } = false;

        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public bool FullRowSelect { get; set; } = true;

        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public ColumnHeaderAutoResizeStyle ColumnResizeStyle { get; set; } = ColumnHeaderAutoResizeStyle.None;

        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public int ItemsPerPage { get; set; } = 20;

        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public ListViewItem SelectedItem
        {
            get
            {
                if (this.SelectedItems.Count > 0)
                    return this.SelectedItems[0];
                return null;
            }
            set
            {
                if (value == null)
                {
                    this.ClearSelection();
                }
                else
                {
                    for (int i = 0; i < this.Items.Count; i++)
                    {
                        if (this.Items[i] == value)
                        {
                            this.SelectedIndex = i;
                            break;
                        }
                    }
                }
            }
        }
        public ListViewBinding()
        {
            base.View = View.Details;
            if (!DesignMode)
            {
                Resize += ListViewBinding_Resize;
                Items.CollectionChanged += Items_CollectionChanged;
            }
        }

        private void Items_CollectionChanged(object sender, CollectionChangeEventArgs e)
        {
            if (e.Action == CollectionChangeAction.Add)
            {
                if (e.Element is ListViewItem lvi)
                {
                    lvi.SubItems.CollectionChanged += Items_CollectionChanged;
                }
                else
                {
                    ListViewItem.ListViewSubItem lvsi = e.Element as ListViewItem.ListViewSubItem;
                    lvi = OwnerProp.GetValue(lvsi, null) as ListViewItem;
                    if (lvi.DataBoundItem != null)
                    {
                        var col = Columns[lvi.SubItems.Count - 1];
                        if (lvi.SubItems.Count == ColumnCount)
                        {
                            lvi.SubItems.CollectionChanged -= Items_CollectionChanged;
                            OnItemBinding(new ListViewItemBindingEventArgs(lvi.DataBoundItem, lvi));
                        }
                    }
                }
            }
        }
        private System.Reflection.PropertyInfo OwnerProp = typeof(ListViewItem.ListViewSubItem).GetProperty("Owner", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic);
        private void ListViewBinding_Resize(object sender, EventArgs e)
        {
            ListView lv = sender as ListView;
            if (lv != null && lv.Columns.Count > 0)
                lv.Columns[lv.Columns.Count - 1].Width = -1;
        }

        private void OnItemBinding(ListViewItemBindingEventArgs e)
        {
            
            this.ItemBinding?.Invoke(this, e);
        }


        public event ListViewItemBindingEventHandler ItemBinding;


        protected override void Dispose(bool disposing)
        {
            Items.CollectionChanged -= Items_CollectionChanged;
            base.Dispose(disposing);
        }
    }
    public class ListViewItemBindingEventArgs
    {
        public ListViewItemBindingEventArgs(object objItem, ListViewItem lvi)
        {
            DataRow = objItem;
            ListViewItem = lvi;
        }

        public ListViewItem ListViewItem { get; set; }
        public object DataRow { set; get; }
    }
    public delegate void ListViewItemBindingEventHandler(object sender, ListViewItemBindingEventArgs e);

    public class ListViewItemEx : ListViewItem
    {
        public ListViewItemEx(string text) : base(text)
        {

        }
        public int DataItemIndex { get; set; }
    }
}
